@@ -1,18 +1,23 @@
 /*
- * xv24to8.c  -  contains the 24-to-8-bit Conv24to8 procedure
+ * xv24to8.c  -  contains the 24-to-8-bit Conv24to8() procedure
+ *               and the 8-to-24-bit Conv8to24() procedure
  *
  * The Conv24to8 procedure takes a pointer to a 24-bit image (loaded
  * previously).  The image will be a w * h * 3 byte array of
  * bytes.  The image will be arranged with 3 bytes per pixel (in order
  * R, G, and B), pixel 0 at the top left corner.  (As normal.)
  * The procedure also takes a maximum number of colors to use (numcols)
+ * and pointers to three 256-long arrays of bytes (to hold the returned
+ * colormap)
  *
- * The Conv24to8 procedure will set up the following:  it will allocate and
- * create 'pic', a pWIDE*pHIGH 8-bit picture.  it will set up pWIDE and pHIGH.
- * it will load up the r[], g[], and b[] colormap arrays.  it will NOT 
+ * Note that Conv24to8() does NOT free the pic24 image under any circumstances
+ *
+ * The Conv24to8 procedure will set up the following:  it will allocate, make
+ * & return 'pic8', a 'w' by 'h' (passed in values) 8-bit picture.
+ * it will load up the rmap, gmap and bmap colormap arrays.  it will NOT 
  * calculate numcols, since the cmap sort procedure has to be called anyway
  *
- * Conv24to8 returns '0' if successful, '1' if it failed (presumably on a 
+ * Conv24to8 returns 'pic8' if successful, 'NULL' on failure (presumably on a 
  * malloc())
  *
  * The 'slow' code is based on Heckbert's Median Cut algorithm.
@@ -22,37 +27,63 @@
  *   Init24to8()
  */
 
-/*
- * Copyright 1989, 1990, 1991, 1992 by John Bradley and
- *                       The University of Pennsylvania
+/* Copyright Notice
+ * ================
+ * Copyright 1989, 1990, 1991, 1992, 1993 by John Bradley
+ * 
+ * Permission to use, copy, and distribute XV in its entirety, for 
+ * non-commercial purposes, is hereby granted without fee, provided that
+ * this license information and copyright notice appear in all copies.
+ * 
+ * Note that distributing XV 'bundled' in with ANY product is considered
+ * to be a 'commercial purpose'.
  *
- * Permission to use, copy, and distribute for non-commercial purposes,
- * is hereby granted without fee, providing that the above copyright
- * notice appear in all copies and that both the copyright notice and this
- * permission notice appear in supporting documentation.
+ * Also note that any copies of XV that are distributed MUST be built
+ * and/or configured to be in their 'unregistered copy' mode, so that it
+ * is made obvious to the user that XV is shareware, and that they should
+ * consider donating, or at least reading this License Info.
+ * 
+ * The software may be modified for your own purposes, but modified
+ * versions may NOT be distributed without prior consent of the author.
+ * 
+ * This software is provided 'as-is', without any express or implied
+ * warranty.  In no event will the author be held liable for any damages
+ * arising from the use of this software.
+ * 
+ * If you would like to do something with XV that this copyright
+ * prohibits (such as distributing it with a commercial product, 
+ * using portions of the source in some other program, etc.), please
+ * contact the author (preferably via email).  Arrangements can
+ * probably be worked out.
  *
- * The software may be modified for your own purposes, but modified versions
- * may not be distributed.
- *
- * This software is provided "as is" without any expressed or implied warranty.
+ * XV is shareware for PERSONAL USE only.  You may use XV for your own
+ * amusement, and if you find it nifty, useful, generally cool, or of
+ * some value to you, your non-deductable donation would be greatly
+ * appreciated.  $25 is the suggested donation, though, of course,
+ * larger donations are quite welcome.  Folks who donate $25 or more
+ * can receive a Real Nice bound copy of the XV manual for no extra
+ * charge.
+ * 
+ * Commercial, government, and institutional users MUST register their
+ * copies of XV, for the exceedingly REASONABLE price of just $25 per
+ * workstation/X terminal.  Site licenses are available for those who
+ * wish to run XV on a large number of machines.  Contact the author
+ * for more details.
  *
  * The author may be contacted via:
- *    US Mail:   John Bradley
- *               GRASP Lab, Room 301C
- *               3401 Walnut St.
- *               Philadelphia, PA  19104
+ *    US Mail:  John Bradley
+ *              1053 Floyd Terrace
+ *              Bryn Mawr, PA  19010
  *
- *    Phone:     (215) 898-8813
- *    EMail:     bradley@cis.upenn.edu
+ *    Phone:    (215) 898-8813
+ *    EMail:    bradley@cis.upenn.edu
  */
 
-
 /*
  * Portions Copyright (C) 1989, 1991 by Jef Poskanzer.  See copyright notice
  * at beginning of relevant code.
  */
 
-
 #include "xv.h"
 
 #define	MAX_CMAP_SIZE	256
@@ -79,24 +110,40 @@ typedef struct {
   int entries[MAX_CMAP_SIZE][2];
 } CCELL;
 
+static byte *pic8;
 static byte *pic24;
+static byte *rmap, *gmap, *bmap;      /* ptrs to computed colormap */
 static int   num_colors, WIDE, HIGH;
 static int   histogram[B_LEN][B_LEN][B_LEN];
 
 CBOX   *freeboxes, *usedboxes;
 CCELL **ColorCells;
 
-static void   get_histogram();
-static CBOX  *largest_box();
-static void   splitbox();
-static void   shrinkbox();
-static void   assign_color();
-static CCELL *create_colorcell();
-static void   map_colortable();
-static int    quant_fsdither();
-static int    Quick24to8();
-static int    QuickCheck();
-static int    ppmquant();
+#ifdef __STDC__
+  static void   get_histogram(CBOX *);
+  static CBOX  *largest_box(void);
+  static void   splitbox(CBOX *);
+  static void   shrinkbox(CBOX *);
+  static void   assign_color(CBOX *, byte *, byte *, byte *);
+  static CCELL *create_colorcell(int, int, int);
+  static void   map_colortable(void);
+  static int    quant_fsdither(void);
+  static int    Quick24to8(byte *, int, int);
+  static int    QuickCheck(byte *, int, int, int);
+  static int    ppmquant(byte *, int, int, int);
+#else
+  static void   get_histogram();
+  static CBOX  *largest_box();
+  static void   splitbox();
+  static void   shrinkbox();
+  static void   assign_color();
+  static CCELL *create_colorcell();
+  static void   map_colortable();
+  static int    quant_fsdither();
+  static int    Quick24to8();
+  static int    QuickCheck();
+  static int    ppmquant();
+#endif
 
 static int    tbl1[512],     /* tables used in F-S Dithering */
               tbl3[512],     /* contain i/16, 3i/16, 5i/16, 7i/16, */
@@ -138,43 +185,53 @@ void Init24to8()
 
 
 /****************************/
-int Conv24to8(p,w,h,nc)
+byte *Conv24to8(p,w,h,nc,rm,gm,bm)
 /****************************/
 byte *p;
+byte *rm, *gm, *bm;
 int   w,h,nc;
 {
   int   i, j;
   CBOX *box_list, *ptr;
 
+  /* returns pointer to new 8-bit-per-pixel image (w*h) if successful, or
+     NULL if unsuccessful */
 
   if (nc<=0) nc = 255;  /* 'nc == 0' breaks code */
 
   /* copy arguments to local-global variables */
-  pic24 = p;  pWIDE = WIDE = w;  pHIGH = HIGH = h;  num_colors = nc;
+  pic24 = p;  WIDE = w;  HIGH = h;  num_colors = nc;
+  rmap = rm;  gmap = gm;  bmap = bm;
+
+  if (!pic24) return NULL;
 
   /* allocate pic immediately, so that if we can't allocate it, we don't
      waste time running this algorithm */
 
-  pic = (byte *) malloc(WIDE * HIGH);
-  if (pic == NULL) {
-    fprintf(stderr,"%s: Conv24to8() - failed to allocate 'pic'\n",cmd);
-    return(1);
+  pic8 = (byte *) malloc(WIDE * HIGH);
+  if (!pic8) {
+    fprintf(stderr,"%s: Conv24to8() - failed to allocate 'pic8'\n",cmd);
+    return pic8;
   }
 
   if (!noqcheck && QuickCheck(pic24,w,h,nc)) { 
     /* see if it's a <256 color RGB pic */
     SetISTR(ISTR_INFO,"No color compression was necessary.\n");
-    return 0;   
+    return pic8;   
   }
 
   else if (conv24 == CONV24_FAST) {
     SetISTR(ISTR_INFO,"Doing 'quick' 24-bit to 8-bit conversion.");
-    return(Quick24to8(pic24,w,h));
+    i = Quick24to8(pic24,w,h);
+    if (i) { free(pic8);  pic8 = NULL; }
+    return pic8;
   }
 
   else if (conv24 == CONV24_BEST) {
     SetISTR(ISTR_INFO,"Doing 'best' 24-bit to 8-bit conversion.");
-    return(ppmquant(pic24,w,h,nc));
+    i = ppmquant(pic24,w,h,nc);
+    if (i) { free(pic8);  pic8 = NULL; }
+    return pic8;
   }
 
   else 
@@ -189,7 +246,8 @@ int   w,h,nc;
 
   if (box_list == NULL) {
     fprintf(stderr,"%s: Conv24to8() - failed to allocate 'freeboxes'\n",cmd);
-    return(1);
+    free(pic8);  pic8 = NULL;
+    return pic8;
   }
 
   for (i=0; i<num_colors; i++) {
@@ -231,7 +289,7 @@ int   w,h,nc;
   /**** STEP 4: assign colors to all boxes ****/
 
   for (i=0, ptr=usedboxes; i<num_colors && ptr; i++, ptr=ptr->next) {
-    assign_color(ptr, &r[i], &g[i], &b[i]);
+    assign_color(ptr, &rmap[i], &gmap[i], &bmap[i]);
   }
 
   /* We're done with the boxes now */
@@ -266,7 +324,8 @@ int   w,h,nc;
   }
   free(ColorCells);
 
-  return i;
+  if (i) { free(pic8);  pic8 = NULL; }  /* free 'pic' on failure */
+  return pic8;
 }
 
 
@@ -572,7 +631,7 @@ static void assign_color(ptr,rp,gp,bp)
      byte *rp,*gp,*bp;
 /*******************************/
 {
-  /* +1 ensures that color represents the middle of the box */
+
   int r,g,b;
 
   r = ((ptr->rmin + ptr->rmax) << (COLOR_DEPTH - B_DEPTH)) / 2;
@@ -613,7 +672,7 @@ static CCELL *create_colorcell(r1,g1,b1)
 
   mindist = 2000000000;
 
-  rp=r;  gp=g;  bp=b;
+  rp=rmap;  gp=gmap;  bp=bmap;
   for (i=0; i<num_colors; i++,rp++,gp++,bp++)
     if( *rp>>(COLOR_DEPTH-C_DEPTH) == ir  &&
         *gp>>(COLOR_DEPTH-C_DEPTH) == ig  &&
@@ -641,7 +700,7 @@ static CCELL *create_colorcell(r1,g1,b1)
 
   /* step 3: find all points within that distance to box */
 
-  rp=r;  gp=g;  bp=b;
+  rp=rmap;  gp=gmap;  bp=bmap;
   for (i=0; i<num_colors; i++,rp++,gp++,bp++)
     if (*rp >> (COLOR_DEPTH-C_DEPTH) != ir  ||
 	*gp >> (COLOR_DEPTH-C_DEPTH) != ig  ||
@@ -722,11 +781,11 @@ static void map_colortable()
 	  dist = 2000000000;
 	  for (i=0; i<cell->num_ents && dist>cell->entries[i][1]; i++) {
 	    j = cell->entries[i][0];
-	    d2 = r[j] - (ir << (COLOR_DEPTH-B_DEPTH));
+	    d2 = rmap[j] - (ir << (COLOR_DEPTH-B_DEPTH));
 	    d2 = (d2 * d2 * R2FACT);
-	    tmp = g[j] - (ig << (COLOR_DEPTH-B_DEPTH));
+	    tmp = gmap[j] - (ig << (COLOR_DEPTH-B_DEPTH));
 	    d2 += (tmp*tmp * G2FACT);
-	    tmp = b[j] - (ib << (COLOR_DEPTH-B_DEPTH));
+	    tmp = bmap[j] - (ib << (COLOR_DEPTH-B_DEPTH));
 	    d2 += (tmp*tmp * B2FACT);
 	    if( d2 < dist ) { dist = d2;  *histp = j; }
 	  }
@@ -743,8 +802,8 @@ static int quant_fsdither()
 {
   register int  *thisptr, *nextptr;
   int           *thisline, *nextline, *tmpptr;
-  int            r1, g1, b1, r2, g2, b2, r3, g3, b3;
-  int            i, j, k, d, imax, jmax, oval;
+  int            r1, g1, b1, r2, g2, b2;
+  int            i, j, imax, jmax, oval;
   byte          *inptr, *outptr;
   int            lastline, lastpixel;
 
@@ -762,7 +821,7 @@ static int quant_fsdither()
 
 
   inptr  = (byte *) pic24;
-  outptr = (byte *) pic;
+  outptr = (byte *) pic8;
 
   /* get first line of picture */
   for (j=WIDE * 3, tmpptr=nextline; j; j--)
@@ -813,11 +872,11 @@ static int quant_fsdither()
 	dist = 2000000000;
 	for (ci=0; ci<cell->num_ents && dist>cell->entries[ci][1]; ci++) {
 	  cj = cell->entries[ci][0];
-	  d2 = (r[cj] >> (COLOR_DEPTH-B_DEPTH)) - r2;
+	  d2 = (rmap[cj] >> (COLOR_DEPTH-B_DEPTH)) - r2;
 	  d2 = (d2*d2 * R2FACT);
-	  tmp = (g[cj] >> (COLOR_DEPTH-B_DEPTH)) - g2;
+	  tmp = (gmap[cj] >> (COLOR_DEPTH-B_DEPTH)) - g2;
 	  d2 += (tmp*tmp * G2FACT);
-	  tmp = (b[cj] >> (COLOR_DEPTH-B_DEPTH)) - b2;
+	  tmp = (bmap[cj] >> (COLOR_DEPTH-B_DEPTH)) - b2;
 	  d2 += (tmp*tmp * B2FACT);
 	  if (d2<dist) { dist = d2;  oval = cj; }
 	}
@@ -826,7 +885,7 @@ static int quant_fsdither()
 
       *outptr++ = oval;
 
-      r1 -= r[oval];  g1 -= g[oval];  b1 -= b[oval];
+      r1 -= rmap[oval];  g1 -= gmap[oval];  b1 -= bmap[oval];
 	
       /* don't use tables, because r1,g1,b1 could go negative */
       if (!lastpixel) {
@@ -882,7 +941,7 @@ int   w,h;
    *
    */
 
-  /* called after 'pic' has been alloced, pWIDE,pHIGH set up, mono/1-bit
+  /* called after 'pic8' has been alloced, pWIDE,pHIGH set up, mono/1-bit
      checked already */
 
   byte *pp;
@@ -891,7 +950,7 @@ int   w,h;
   int  i, j, val, pwide3;
   int  imax, jmax;
 
-  pp = pic;  pwide3 = w * 3;  imax = h-1;  jmax = w-1;
+  pp = pic8;  pwide3 = w * 3;  imax = h-1;  jmax = w-1;
 
   /* up to 256 colors:     3 bits R, 3 bits G, 2 bits B  (RRRGGGBB) */
 #define RMASK 0xe0
@@ -905,9 +964,9 @@ int   w,h;
   /* note that 0 and 255 of each color are always in the map; */
   /* intermediate values are evenly spaced. */
   for (i=0; i<256; i++) {
-    r[i] = (((i<<R_SHIFT) & RMASK) * 255 + RMASK/2) / RMASK;
-    g[i] = (((i<<G_SHIFT) & GMASK) * 255 + GMASK/2) / GMASK;
-    b[i] = (((i<<B_SHIFT) & BMASK) * 255 + BMASK/2) / BMASK;
+    rmap[i] = (((i<<R_SHIFT) & RMASK) * 255 + RMASK/2) / RMASK;
+    gmap[i] = (((i<<G_SHIFT) & GMASK) * 255 + GMASK/2) / GMASK;
+    bmap[i] = (((i<<B_SHIFT) & BMASK) * 255 + BMASK/2) / BMASK;
   }
 
 
@@ -940,9 +999,9 @@ int   w,h;
       *pp = val;
 
       /* compute color errors */
-      r1 -= r[val];
-      g1 -= g[val];
-      b1 -= b[val];
+      r1 -= rmap[val];
+      g1 -= gmap[val];
+      b1 -= bmap[val];
 
       /* Add fractions of errors to adjacent pixels */
       r1 += 256;              /* make positive for table indexing */
@@ -990,7 +1049,7 @@ int   w,h,maxcol;
      finds more than 'maxcol' colors, it returns '0'.  If it DOESN'T, it does
      the 24-to-8 conversion by simply sticking the colors it found into
      a colormap, and changing instances of a color in pic24 into colormap
-     indicies (in pic) */
+     indicies (in pic8) */
 
   unsigned long colors[256],col;
   int           i, nc, low, high, mid;
@@ -1016,10 +1075,9 @@ int   w,h,maxcol;
     }
 
     if (high < low) { /* didn't find color in list, add it. */
-      /* WARNING: this is an overlapped memory copy.  memcpy doesn't do
-	 it correctly, hence 'bcopy', which claims to */
       if (nc>=maxcol) return 0;
-      bcopy(&colors[low], &colors[low+1], (nc - low) * sizeof(unsigned long));
+      xvbcopy((char *) &colors[low], (char *) &colors[low+1],
+	      (nc - low) * sizeof(u_long));
       colors[low] = col;
       nc++;
     }
@@ -1029,7 +1087,7 @@ int   w,h,maxcol;
   /* run through the data a second time, this time mapping pixel values in
      pic24 into colormap offsets into 'colors' */
 
-  for (i=w*h,p=pic24, pix=pic; i; i--,pix++) {
+  for (i=w*h,p=pic24, pix=pic8; i; i--,pix++) {
     col  = (((u_long) *p++) << 16);  
     col += (((u_long) *p++) << 8);
     col +=  *p++;
@@ -1052,9 +1110,9 @@ int   w,h,maxcol;
 
   /* and load up the 'desired colormap' */
   for (i=0; i<nc; i++) {
-    r[i] =  colors[i]>>16;  
-    g[i] = (colors[i]>>8) & 0xff;
-    b[i] =  colors[i]     & 0xff;
+    rmap[i] =  colors[i]>>16;  
+    gmap[i] = (colors[i]>>8) & 0xff;
+    bmap[i] =  colors[i]     & 0xff;
   }
 
   return 1;
@@ -1062,8 +1120,34 @@ int   w,h,maxcol;
 
 
 
+/***************************************************************/
+byte *Conv8to24(pic8, w, h, rmap,gmap,bmap)
+     byte *pic8, *rmap, *gmap, *bmap;
+     int   w, h;
+{
+  /* converts an w*h 8-bit image (with colormap rmap,gmap,bmap) into a
+   * 24-bit image.  Note, 'pic8' could be NULL
+   *
+   * returns pointer to new 24-bits-per-pixel image (w*h) if successful,
+   * or NULL if unsuccessful
+   */
 
+  int   i;
+  byte *pic24, *sp, *dp;
 
+  pic24 = (byte *) malloc(w * h * 3);
+  if (!pic24) return pic24;
+
+  for (i=w*h, sp=pic8, dp=pic24; i; i--, sp++) {
+    if ((i&0x1ffff)==0) WaitCursor();
+    *dp++ = rmap[*sp];
+    *dp++ = gmap[*sp];
+    *dp++ = bmap[*sp];
+  }
+
+  return pic24;
+}
+  
 
 
 
@@ -1148,6 +1232,13 @@ struct colorhist_list_item { struct colorhist_item ch;
 
 typedef colorhist_list* colorhash_table;
 
+typedef struct box* box_vector;
+struct box {
+  int index;
+  int colors;
+  int sum;
+};
+
 
 #define MAXCOLORS 32767
 #define CLUSTER_MAXVAL 63
@@ -1167,38 +1258,48 @@ typedef colorhist_list* colorhash_table;
 
 
 /*** function defs ***/
-static colorhist_vector mediancut();
-static int              redcompare(), greencompare(), bluecompare();
-static int              sumcompare();
-static colorhist_vector ppm_computecolorhist(), ppm_colorhashtocolorhist();
-static colorhash_table  ppm_computecolorhash(), ppm_alloccolorhash();
-static void             ppm_addtocolorhash();
-static void             ppm_freecolorhist(), ppm_freecolorhash();
-static int              ppm_lookupcolor();
+
+#ifdef __STDC__
+  static colorhist_vector mediancut(colorhist_vector, int, int, int, int);
+  static int              redcompare  (colorhist_vector, colorhist_vector);
+  static int              greencompare(colorhist_vector, colorhist_vector);
+  static int              bluecompare (colorhist_vector, colorhist_vector);
+  static int              sumcompare  (box_vector, box_vector);
+  static colorhist_vector ppm_computecolorhist(pixel **, int,int,int,int *);
+  static colorhash_table  ppm_computecolorhash(pixel **, int,int,int,int *);
+  static colorhist_vector ppm_colorhashtocolorhist(colorhash_table, int);
+  static colorhash_table  ppm_alloccolorhash(void);
+  static void             ppm_freecolorhist(colorhist_vector);
+  static void             ppm_freecolorhash(colorhash_table);
+#else
+  static colorhist_vector mediancut();
+  static int              redcompare(), greencompare(), bluecompare();
+  static int              sumcompare();
+  static colorhist_vector ppm_computecolorhist(), ppm_colorhashtocolorhist();
+  static colorhash_table  ppm_computecolorhash(), ppm_alloccolorhash();
+  static void             ppm_freecolorhist(), ppm_freecolorhash();
+#endif
 
 
 /****************************************************************************/
 static int ppmquant(pic24, cols, rows, newcolors)
-byte *pic24;
-int  cols, rows, newcolors;
+     byte *pic24;
+     int  cols, rows, newcolors;
 {
   pixel**          pixels;
   register pixel*  pP;
-  int              argn, row;
+  int              row;
   register int     col, limitcol;
   pixval           maxval, newmaxval;
   int              colors;
   register int     index;
   colorhist_vector chv, colormap;
   colorhash_table  cht;
-  int              verbose, floyd;
-  long             *thisrerr, *nextrerr, *thisgerr, *nextgerr;
-  long             *thisberr, *nextberr, *temperr;
-  register long    sr, sg, sb, err;
-  int              fs_direction, i;
+  int              i;
   unsigned char    *picptr;
   static char      *fn = "ppmquant()";
 
+  index = 0;
   maxval = 255;
 
   /*
@@ -1274,7 +1375,7 @@ int  cols, rows, newcolors;
   if (DEBUG) fprintf(stderr,"%s: mapping image to new colors\n", fn);
   cht = ppm_alloccolorhash();
 
-  picptr = pic;
+  picptr = pic8;
   for (row = 0;  row < rows;  ++row) {
     col = 0;  limitcol = cols;  pP = pixels[row];
 
@@ -1284,7 +1385,6 @@ int  cols, rows, newcolors;
       colorhist_list chl;
 
       /* Check hash table to see if we have already matched this color. */
-      /* index = ppm_lookupcolor( cht, pP ); */
 
       hash = ppm_hashpixel(*pP);
       for (chl = cht[hash];  chl;  chl = chl->next)
@@ -1311,7 +1411,6 @@ int  cols, rows, newcolors;
 	  if (newdist<dist) { index = i;  dist = newdist; }
 	}
 
-	/* ppm_addtocolorhash(cht, pP, index); */
 	hash = ppm_hashpixel(*pP);
 	chl = (colorhist_list) malloc(sizeof(struct colorhist_list_item));
 	if (!chl) FatalError("ran out of memory adding to hash table");
@@ -1333,9 +1432,9 @@ int  cols, rows, newcolors;
   /* rescale the colormap and load the XV colormap */
   for (i=0; i<newcolors; i++) {
     PPM_DEPTH(colormap[i].color, colormap[i].color, maxval, 255);
-    r[i] = PPM_GETR( colormap[i].color );
-    g[i] = PPM_GETG( colormap[i].color );
-    b[i] = PPM_GETB( colormap[i].color );
+    rmap[i] = PPM_GETR( colormap[i].color );
+    gmap[i] = PPM_GETG( colormap[i].color );
+    bmap[i] = PPM_GETB( colormap[i].color );
   }
 
   /* free the pixels array */
@@ -1357,19 +1456,13 @@ int  cols, rows, newcolors;
 ** Display", SIGGRAPH '82 Proceedings, page 297.
 */
 
-typedef struct box* box_vector;
-struct box {
-  int index;
-  int colors;
-  int sum;
-};
 
 
 /****************************************************************************/
 static colorhist_vector mediancut( chv, colors, sum, maxval, newcolors )
      colorhist_vector chv;
      int colors, sum, newcolors;
-     pixval maxval;
+     int maxval;
 {
   colorhist_vector colormap;
   box_vector bv;
@@ -1530,28 +1623,28 @@ static colorhist_vector mediancut( chv, colors, sum, maxval, newcolors )
 
 /**********************************/
 static int redcompare(ch1, ch2)
-colorhist_vector ch1, ch2;
+     colorhist_vector ch1, ch2;
 {
   return (int) PPM_GETR( ch1->color ) - (int) PPM_GETR( ch2->color );
 }
 
 /**********************************/
 static int greencompare(ch1, ch2)
-colorhist_vector ch1, ch2;
+     colorhist_vector ch1, ch2;
 {
   return (int) PPM_GETG( ch1->color ) - (int) PPM_GETG( ch2->color );
 }
 
 /**********************************/
 static int bluecompare( ch1, ch2 )
-colorhist_vector ch1, ch2;
+     colorhist_vector ch1, ch2;
 {
   return (int) PPM_GETB( ch1->color ) - (int) PPM_GETB( ch2->color );
 }
 
 /**********************************/
 static int sumcompare( b1, b2 )
-box_vector b1, b2;
+     box_vector b1, b2;
 {
   return b2->sum - b1->sum;
 }
@@ -1559,10 +1652,10 @@ box_vector b1, b2;
 
 
 /****************************************************************************/
-static colorhist_vector  ppm_computecolorhist(pixels, cols, rows, 
-					      maxcolors, colorsP)
+static colorhist_vector 
+  ppm_computecolorhist(pixels, cols, rows, maxcolors, colorsP)
      pixel** pixels;
-     int cols, rows;
+     int cols, rows, maxcolors;
      int* colorsP;
 {
   colorhash_table cht;
@@ -1581,7 +1674,7 @@ static colorhist_vector  ppm_computecolorhist(pixels, cols, rows,
 static colorhash_table ppm_computecolorhash(pixels, cols, rows, 
 					    maxcolors, colorsP )
      pixel** pixels;
-     int cols, rows;
+     int cols, rows, maxcolors;
      int* colorsP;
 {
   colorhash_table cht;
@@ -1637,26 +1730,6 @@ static colorhash_table ppm_alloccolorhash()
 }
 
 
-/****************************************************************************/
-static void ppm_addtocolorhash( cht, colorP, value )
-     colorhash_table cht;
-     pixel* colorP;
-     int value;
-{
-  int hash;
-  colorhist_list chl;
-
-  hash = ppm_hashpixel( *colorP );
-  chl = (colorhist_list) malloc( sizeof(struct colorhist_list_item) );
-  if (!chl) FatalError("ran out of memory adding to hash table");
-
-  chl->ch.color = *colorP;
-  chl->ch.value = value;
-  chl->next = cht[hash];
-  cht[hash] = chl;
-}
-
-
 /****************************************************************************/
 static colorhist_vector ppm_colorhashtocolorhist( cht, maxcolors )
      colorhash_table cht;
@@ -1685,22 +1758,6 @@ static colorhist_vector ppm_colorhashtocolorhist( cht, maxcolors )
 }
 
 
-/****************************************************************************/
-static int ppm_lookupcolor( cht, colorP )
-     colorhash_table cht;
-     pixel* colorP;
-{
-  int hash;
-  colorhist_list chl;
-
-  hash = ppm_hashpixel( *colorP );
-  for (chl = cht[hash];  chl != (colorhist_list) 0;  chl = chl->next)
-    if (PPM_EQUAL(chl->ch.color, *colorP)) return chl->ch.value;
-
-  return -1;
-}
-
-
 /****************************************************************************/
 static void ppm_freecolorhist( chv )
      colorhist_vector chv;
