@@ -15,28 +15,56 @@
 
 /* WriteJFIF() and JPEG dialog routines written by John Bradley */
 
-/*
- * Portions Copyright 1989, 1990, 1991, 1992 by John Bradley and
- *                                The University of Pennsylvania
+/* Copyright Notice
+ * ================
+ * Portions Copyright 1989, 1990, 1991, 1992, 1993 by John Bradley
+ * 
+ * Permission to use, copy, and distribute XV in its entirety, for 
+ * non-commercial purposes, is hereby granted without fee, provided that
+ * this license information and copyright notice appear in all copies.
+ * 
+ * Note that distributing XV 'bundled' in with ANY product is considered
+ * to be a 'commercial purpose'.
  *
- * Permission to use, copy, and distribute for non-commercial purposes,
- * is hereby granted without fee, providing that the above copyright
- * notice appear in all copies and that both the copyright notice and this
- * permission notice appear in supporting documentation. 
+ * Also note that any copies of XV that are distributed MUST be built
+ * and/or configured to be in their 'unregistered copy' mode, so that it
+ * is made obvious to the user that XV is shareware, and that they should
+ * consider donating, or at least reading this License Info.
+ * 
+ * The software may be modified for your own purposes, but modified
+ * versions may NOT be distributed without prior consent of the author.
+ * 
+ * This software is provided 'as-is', without any express or implied
+ * warranty.  In no event will the author be held liable for any damages
+ * arising from the use of this software.
+ * 
+ * If you would like to do something with XV that this copyright
+ * prohibits (such as distributing it with a commercial product, 
+ * using portions of the source in some other program, etc.), please
+ * contact the author (preferably via email).  Arrangements can
+ * probably be worked out.
  *
- * The software may be modified for your own purposes, but modified versions
- * may not be distributed.
- *
- * This software is provided "as is" without any expressed or implied warranty.
+ * XV is shareware for PERSONAL USE only.  You may use XV for your own
+ * amusement, and if you find it nifty, useful, generally cool, or of
+ * some value to you, your non-deductable donation would be greatly
+ * appreciated.  $25 is the suggested donation, though, of course,
+ * larger donations are quite welcome.  Folks who donate $25 or more
+ * can receive a Real Nice bound copy of the XV manual for no extra
+ * charge.
+ * 
+ * Commercial, government, and institutional users MUST register their
+ * copies of XV, for the exceedingly REASONABLE price of just $25 per
+ * workstation/X terminal.  Site licenses are available for those who
+ * wish to run XV on a large number of machines.  Contact the author
+ * for more details.
  *
  * The author may be contacted via:
- *    US Mail:   John Bradley
- *               GRASP Lab, Room 301C
- *               3401 Walnut St.  
- *               Philadelphia, PA  19104
+ *    US Mail:  John Bradley
+ *              1053 Floyd Terrace
+ *              Bryn Mawr, PA  19010
  *
- *    Phone:     (215) 898-8813
- *    EMail:     bradley@cis.upenn.edu       
+ *    Phone:    (215) 898-8813
+ *    EMail:    bradley@cis.upenn.edu
  */
 
 
@@ -44,59 +72,71 @@
 #include "jinclude.h"
 #include <setjmp.h>
 
+#define CREATOR_STR "CREATOR: "
+
 
 /**** Stuff for JPEGDialog box ****/
 
-#define JWIDE 280
-#define JHIGH 160
+#define JWIDE 400
+#define JHIGH 200
 #define J_NBUTTS 2
 #define J_BOK    0
 #define J_BCANC  1
 #define BUTTH    24
 
 #ifdef __STDC__
-static void drawJD(int, int, int, int);
-static void clickJD(int, int);
-static void doCmd(int);
-static void writeJPEG(void);
+  static void drawJD(int, int, int, int);
+  static void clickJD(int, int);
+  static void doCmd(int);
+  static void writeJPEG(void);
+  static void jselwxv(decompress_info_ptr);
+  static int  writeJFIF(FILE *);
 #else
-static void drawJD(), doCmd(), clickJD(), writeJPEG();
+  static void drawJD(), doCmd(), clickJD(), writeJPEG();
+  static void jselwxv();
+  static int writeJFIF();
 #endif
-static void jselwxv();
-static int writeJFIF();
+
 
 
 /* local variables */
 static char *filename;
 static int   colorType;
-static DIAL  qDial;
+static DIAL  qDial, smDial;
 static BUTT  jbut[J_NBUTTS];
 static byte *image8, *image24;
+static char *loadComments;
 
+static int   Width, Height;
 static byte        *CurImagePtr;
-static byte        *pic24;
+static byte        *pic24, *pic8, *rmap, *gmap, *bmap;
 static long  	    filesize;
 static jmp_buf      jmpState;
-static struct external_methods_struct   e_methods;
+static struct External_methods_struct   e_methods;
 
 
 /***************************************************/
 void CreateJPEGW()
 {
-  jpegW = CreateWindow("xv jpeg", NULL, JWIDE, JHIGH, infofg, infobg);
+  XClassHint classh;
+
+  jpegW = CreateWindow("xv jpeg", "XVjpeg", NULL, 
+		       JWIDE, JHIGH, infofg, infobg, 0);
   if (!jpegW) FatalError("can't create jpeg window!");
-  StoreDeleteWindowProp(jpegW);
 
   XSelectInput(theDisp, jpegW, ExposureMask | ButtonPressMask | KeyPressMask);
 
-  DCreate(&qDial, jpegW, 190-2, 10, 80, 100, 1, 100, 75, 5, 
-	  infofg, infobg, "Quality", "%");
+  DCreate(&qDial, jpegW, 10, 10, 80, 100, 1, 100, 75, 5, 
+	  infofg, infobg, hicol, locol, "Quality", "%");
 
-  BTCreate(&jbut[J_BOK], jpegW, JWIDE-140-1, JHIGH-10-BUTTH-1, 60, BUTTH, 
-	   "Ok", infofg, infobg);
+  DCreate(&smDial, jpegW, 120, 10, 80, 100, 0, 100, 0, 5, 
+	  infofg, infobg, hicol, locol, "Smoothing", "%");
 
-  BTCreate(&jbut[J_BCANC], jpegW, JWIDE-70-1, JHIGH-10-BUTTH-1, 60, BUTTH, 
-	   "Cancel", infofg, infobg);
+  BTCreate(&jbut[J_BOK], jpegW, JWIDE-180-1, JHIGH-10-BUTTH-1, 80, BUTTH, 
+	   "Ok", infofg, infobg, hicol, locol);
+
+  BTCreate(&jbut[J_BCANC], jpegW, JWIDE-90-1, JHIGH-10-BUTTH-1, 80, BUTTH, 
+	   "Cancel", infofg, infobg, hicol, locol);
 
   XMapSubwindows(theDisp, jpegW);
 }
@@ -133,10 +173,12 @@ XEvent *xev;
     x = e->x;  y = e->y;  w = e->width;  h = e->height;
 
     /* throw away excess expose events for 'dumb' windows */
-    if (e->count > 0 && e->window == qDial.win) {}
+    if (e->count > 0 && (e->window == qDial.win || 
+			 e->window == smDial.win)) {}
 
     else if (e->window == jpegW)       drawJD(x, y, w, h);
     else if (e->window == qDial.win)   DRedraw(&qDial);
+    else if (e->window == smDial.win)  DRedraw(&smDial);
     else rv = 0;
   }
 
@@ -146,8 +188,9 @@ XEvent *xev;
     x = e->x;  y = e->y;
 
     if (e->button == Button1) {
-      if      (e->window == jpegW)     clickJD(x,y);
-      else if (e->window == qDial.win) DTrack(&qDial, x,y);
+      if      (e->window == jpegW)      clickJD(x,y);
+      else if (e->window == qDial.win)  DTrack(&qDial,  x,y);
+      else if (e->window == smDial.win) DTrack(&smDial, x,y);
       else rv = 0;
     }  /* button1 */
     else rv = 0;
@@ -200,9 +243,19 @@ static void drawJD(x,y,w,h)
 int x,y,w,h;
 {
   char *title  = "Save JPEG file...";
-  char *title1 = "Set quality value.";
-  char *title2 = "Default value: 75%";
-  char *title3 = "Useful range 5-95%.";
+  char *title1 = "Quality value determines";
+  char *title2 = "compression rate: higher";
+  char *title3 = "quality = bigger file.";
+  char *title4 = "Use smoothing if saving";
+  char *title5 = "an 8-bit image (eg, a GIF).";
+  
+  char *qtitle1 = "Default = 75.";
+  char *qtitle2 = "Useful range";
+  char *qtitle3 = "is 5-95.";
+  char *smtitle1 = "Default = 0 (none).";
+  char *smtitle2 = "10-30 is enough";
+  char *smtitle3 = "for typical GIFs.";
+  
   int  i;
   XRectangle xr;
 
@@ -214,12 +267,34 @@ int x,y,w,h;
 
   for (i=0; i<J_NBUTTS; i++) BTRedraw(&jbut[i]);
 
-  XDrawString(theDisp, jpegW, theGC, 20, 29, title, strlen(title));
 
-  XDrawString(theDisp, jpegW, theGC, 30, 29+LINEHIGH*2,title1,strlen(title1));
-  XDrawString(theDisp, jpegW, theGC, 30, 29+LINEHIGH*4,title2,strlen(title2));
-  XDrawString(theDisp, jpegW, theGC, 30, 29+LINEHIGH*5,title3,strlen(title3));
+  XDrawString(theDisp, jpegW, theGC, 220, 10+ASCENT, 
+	      title, strlen(title));
+  XDrawString(theDisp, jpegW, theGC, 230, 10+ASCENT+LINEHIGH*1,
+	      title1,strlen(title1));
+  XDrawString(theDisp, jpegW, theGC, 230, 10+ASCENT+LINEHIGH*2,
+	      title2,strlen(title2));
+  XDrawString(theDisp, jpegW, theGC, 230, 10+ASCENT+LINEHIGH*3,
+	      title3,strlen(title3));
+  XDrawString(theDisp, jpegW, theGC, 230, 10+ASCENT+LINEHIGH*4,
+	      title4,strlen(title4));
+  XDrawString(theDisp, jpegW, theGC, 230, 10+ASCENT+LINEHIGH*5,
+	      title5,strlen(title5));
 
+  XDrawString(theDisp, jpegW, theGC, 15, 10+100+10+ASCENT,
+	      qtitle1,strlen(qtitle1));
+  XDrawString(theDisp, jpegW, theGC, 15, 10+100+10+ASCENT+LINEHIGH,
+	      qtitle2,strlen(qtitle2));
+  XDrawString(theDisp, jpegW, theGC, 15, 10+100+10+ASCENT+LINEHIGH*2,
+	      qtitle3,strlen(qtitle3));
+  
+  XDrawString(theDisp, jpegW, theGC, 115, 10+100+10+ASCENT+LINEHIGH*0,
+	      smtitle1,strlen(smtitle1));
+  XDrawString(theDisp, jpegW, theGC, 115, 10+100+10+ASCENT+LINEHIGH*1,
+	      smtitle2,strlen(smtitle2));
+  XDrawString(theDisp, jpegW, theGC, 115, 10+100+10+ASCENT+LINEHIGH*2,
+	      smtitle3,strlen(smtitle3));
+  
   XSetClipMask(theDisp, theGC, None);
 }
 
@@ -264,9 +339,9 @@ int cmd;
 static void writeJPEG()
 {
   FILE *fp;
-  int   i, nc, rv, w, h;
+  int   i, nc, rv, w, h, ptype;
   register byte *ip, *ep;
-  byte *inpix, *bwpic, *rmap, *gmap, *bmap;
+  byte *inpix, *bwpic, *rmap, *gmap, *bmap, *gampic24;
 
   /* get the XV image into a format that the JPEG software can grok on.
      Also, open the output file, so we don't waste time doing this format
@@ -275,8 +350,9 @@ static void writeJPEG()
   if (savenormCB.val) { inpix = cpic;  w = cWIDE;  h = cHIGH; }
                  else { inpix = epic;  w = eWIDE;  h = eHIGH; }
 
-  bwpic = NULL;  rmap = r;  gmap = g;  bmap = b;
+  bwpic = NULL;  rmap = rMap;  gmap = gMap;  bmap = bMap;
   nc = numcols;
+  ptype = picType;
 
   /* see if we can open the output file before proceeding */
   fp = OpenOutFile(filename);
@@ -285,13 +361,27 @@ static void writeJPEG()
   WaitCursor();
   
   bwpic = HandleBWandReduced(colorType, &nc, &rmap, &gmap, &bmap);
-  if (bwpic) inpix = bwpic;
+  if (bwpic) { inpix = bwpic;  ptype = PIC8; }
+
+
+  /* deal with possiblity of Gamma-fication in 24-bit mode... */
+  gampic24 = NULL;
+  if (ptype == PIC24) {
+    gampic24 = GammifyPic24(inpix, w, h);
+    if (gampic24) inpix = gampic24;
+  }
+
 
   /* monocity check.  if the image is mono, save it that way to save space */
-
   if (colorType != F_GREYSCALE) {
-    for (i=0; i<nc && rmap[i]==gmap[i] && rmap[i]==bmap[i]; i++);
-    if (i==nc) colorType = F_GREYSCALE;    /* made it all the way through */
+    if (ptype == PIC8) {
+      for (i=0; i<nc && rmap[i]==gmap[i] && rmap[i]==bmap[i]; i++);
+      if (i==nc) colorType = F_GREYSCALE;    /* made it all the way through */
+    }
+    else {  /* PIC24 */
+      for (i=0,ip=inpix; i<w*h && ip[0]==ip[1] && ip[1]==ip[2]; i++,ip+=3);
+      if (i==w*h) colorType = F_GREYSCALE;  /* all the way through */
+    }
   }
     
 
@@ -302,15 +392,18 @@ static void writeJPEG()
     image8 = (byte *) malloc(w * h);
     if (!image8) FatalError("writeJPEG: unable to malloc image8\n");
 
-    for (i=0,ip=image8,ep=inpix; i<w * h; i++, ip++, ep++)
-      *ip = MONO(rmap[*ep], gmap[*ep], bmap[*ep]);
+    if (ptype == PIC8) {
+      for (i=0,ip=image8,ep=inpix; i<w * h; i++, ip++, ep++)
+	*ip = MONO(rmap[*ep], gmap[*ep], bmap[*ep]);
+    }
+    else {  /* PIC24 */
+      for (i=0,ip=image8,ep=inpix; i<w*h; i++, ip++, ep+=3)
+	*ip = MONO(ep[0],ep[1],ep[2]);
+    }
   }
 
   else {    /* colorType = some color format */
-    image24 = NULL;
-    /* if fullcolor, and saving at current size, 'smooth' it */
-    if (colorType == F_FULLCOLOR && !savenormCB.val) image24 = Smooth24();
-    if (!image24) {  /* smoothing didn't/not tried.  Use 'stupid' method */
+    if (ptype == PIC8) {
       image24 = (byte *) malloc(w * h * 3);
       if (!image24) {  /* this simply isn't going to work */
 	FatalError("writeJPEG: unable to malloc image24\n");
@@ -322,6 +415,10 @@ static void writeJPEG()
 	*ip++ = bmap[*ep];
       }
     }
+
+    else {  /* PIC24 */
+      image24 = inpix;
+    }
   }
 
   if (bwpic) free(bwpic);  /* won't need it anymore */
@@ -331,19 +428,23 @@ static void writeJPEG()
 
   /* get rid of the greyscale/truecolor image */
   if (colorType == F_GREYSCALE) free(image8);
-                           else free(image24);
+  else if (ptype==PIC8) free(image24);
 
   if (CloseOutFile(fp, filename, rv) == 0) {
     /* everything's cool! */
     DirBox(0);
-    LoadCurrentDirectory();
   }
 
+  if (gampic24) free(gampic24);
+
+  SetCursors(-1);
 }
 
 
 
 
+
+
 /*********************************************/
 /**** INTERFACE CODE FOR THE JPEG LIBRARY ****/
 /*********************************************/
@@ -357,23 +458,60 @@ static void xv_jpeg_monitor(cinfo, loopcnt, looplimit)
      decompress_info_ptr cinfo;
      long loopcnt, looplimit;
 {
-  int a,b;
-  double percent;
-
   WaitCursor();
 
-#ifdef FOO  
-  a = cinfo->completed_passes;
-  b = cinfo->total_passes;
+#ifdef FOO
+  {
+    int a,b;
+    double percent;
 
-  percent = ((a + ((double) loopcnt / looplimit)) / (double) b) * 100.0;
+    a = cinfo->completed_passes;
+    b = cinfo->total_passes;
 
-  fprintf(stderr,"jpeg: %lf done.  loop: %ld, %ld  pass: %d, %d\n",
-	  percent, loopcnt, looplimit, a, b);
+    percent = ((a + ((double) loopcnt / looplimit)) / (double) b) * 100.0;
+
+    fprintf(stderr,"jpeg: %lf done.  loop: %ld, %ld  pass: %d, %d\n",
+	    percent, loopcnt, looplimit, a, b);
+  }
 #endif
 }
 
 
+/**************************************************/
+static void xv_process_comment(cinfo, length)
+     decompress_info_ptr cinfo;
+     long                length;
+{
+  char *sp, *oldsp;
+  int   ch,hasnull;
+
+
+  if (!loadComments) {
+    loadComments = (char *) malloc(length + 1);
+    if (loadComments) loadComments[0] = '\0';
+  }
+  else {
+    loadComments = (char *) realloc(loadComments, 
+				    strlen(loadComments) + length + 1);
+  }
+  
+  if (!loadComments) FatalError("out of memory in xv_process_comment");
+
+  oldsp = sp = loadComments + strlen(loadComments);
+
+  hasnull = 0;
+  while (length-- > 0) {
+    ch = JGETC(cinfo);
+    *sp++ = (char) ch;
+    if (ch==0) hasnull = 1;
+  }
+
+  if (hasnull) sp = oldsp;     /* swallow comments that have nulls in them */
+
+  *sp++ = '\0';
+}
+
+
 /**************************************************/
 static void d_ui_method_selection(cinfo)
      decompress_info_ptr cinfo;
@@ -384,29 +522,21 @@ static void d_ui_method_selection(cinfo)
   if (cinfo->jpeg_color_space == CS_GRAYSCALE) {
     cinfo->out_color_space = CS_GRAYSCALE;
     cinfo->quantize_colors = FALSE;
-    SetISTR(ISTR_FORMAT,"Greyscale JPEG. (%ld bytes)", filesize);
+
+    SetISTR(ISTR_INFO,"Loading %ldx%ld Greyscale JPEG (%ld bytes)...",
+	    cinfo->image_width, cinfo->image_height, filesize);
+
     /* fill in a greyscale colormap */
-    for (i=0; i<=255; i++)
-      r[i] = g[i] = b[i] = i;
+    for (i=0; i<256; i++) rmap[i] = gmap[i] = bmap[i] = i;
   }
 
   else {
     cinfo->out_color_space = CS_RGB;
+    cinfo->quantize_colors = FALSE;   /* give 24-bit image to XV */
 
-    if (conv24 == CONV24_FAST || conv24 == CONV24_SLOW)
-         cinfo->quantize_colors = TRUE;
-    else cinfo->quantize_colors = FALSE;
+    SetISTR(ISTR_INFO,"Loading %ldx%ld Color JPEG (%ld bytes)...",
+	    cinfo->image_width, cinfo->image_height, filesize);
 
-    if (conv24 != CONV24_FAST) cinfo->two_pass_quantize = TRUE;
-    else {
-      cinfo->two_pass_quantize = FALSE;
-      /* For the 1-pass quantizer it's best to use 256 colors and let */
-      /* the cmap sorter pick up any slack.  This produces essentially */
-      /* the same results as xv's usual fast quantizer, but it reduces */
-      /* swapping since no 24-bit copy of the image is needed. */
-      cinfo->desired_number_of_colors = 256;
-    }
-    SetISTR(ISTR_FORMAT,"Color JPEG. (%ld bytes)", filesize);
   }
 
   jselwxv(cinfo);
@@ -417,18 +547,17 @@ static void d_ui_method_selection(cinfo)
 static void output_init (cinfo)
      decompress_info_ptr cinfo;
 {
-  pWIDE = cinfo->image_width;
-  pHIGH = cinfo->image_height;
+  Width  = cinfo->image_width;
+  Height = cinfo->image_height;
 
-  if (cinfo->out_color_space == CS_GRAYSCALE || 
-      cinfo->quantize_colors == TRUE) {
-    pic = (byte *) malloc(pWIDE * pHIGH);
-    if (!pic) FatalError("Not enough memory for Image");
-    CurImagePtr = pic;
+  if (cinfo->out_color_space == CS_GRAYSCALE) {
+    pic8 = (byte *) malloc(Width * Height);
+    if (!pic8) FatalError("Not enough memory for Image");
+    CurImagePtr = pic8;
   }
 
   else {
-    pic24 = (byte *) malloc(pWIDE * pHIGH * 3);
+    pic24 = (byte *) malloc(Width * Height * 3);
     if (!pic24) FatalError("Not enough memory for Image");
     CurImagePtr = pic24;
   }
@@ -444,9 +573,9 @@ static void put_color_map (cinfo, num_colors, colormap)
   int i;
 
   for (i = 0; i < num_colors; i++) {
-    r[i] = GETJSAMPLE(colormap[0][i]);
-    g[i] = GETJSAMPLE(colormap[1][i]);
-    b[i] = GETJSAMPLE(colormap[2][i]);
+    rmap[i] = GETJSAMPLE(colormap[0][i]);
+    gmap[i] = GETJSAMPLE(colormap[1][i]);
+    bmap[i] = GETJSAMPLE(colormap[2][i]);
   }
 }
 
@@ -523,7 +652,7 @@ static void JPEG_Message (msgtext)
 	  e_methods.message_parm[2], e_methods.message_parm[3],
 	  e_methods.message_parm[4], e_methods.message_parm[5],
 	  e_methods.message_parm[6], e_methods.message_parm[7]);
-  SetISTR(ISTR_INFO, tempstr);
+  SetISTR(ISTR_WARNING, tempstr);
 }
 
 
@@ -544,76 +673,115 @@ static void JPEG_Error (msgtext)
 }
 
 
+
+
+
+
+
+
 /*******************************************/
-int LoadJFIF(fname,nc)
-     char *fname;
-     int   nc;
+int LoadJFIF(fname, pinfo)
+     char    *fname;
+     PICINFO *pinfo;
 /*******************************************/
 {
+  /* returns '1' on success, '0' on failure */
   int rtval;
-  struct decompress_info_struct    cinfo;
-  struct decompress_methods_struct dc_methods;
+  struct Decompress_info_struct    cinfo;
+  struct Decompress_methods_struct dc_methods;
 
   /* Set up the input file */
+  pinfo->type = PIC8;
 
-  if ((cinfo.input_file = fopen(fname, "r")) == NULL) return 1;
+#ifdef VMS
+  if ((cinfo.input_file = fopen(fname, "r", "ctx=stm")) == NULL) return 0;
+#else
+  if ((cinfo.input_file = fopen(fname, "r")) == NULL) return 0;
+#endif
 
-  /* figure out the file size (for Informational Purposes Only) */
   fseek(cinfo.input_file, 0L, 2);
   filesize = ftell(cinfo.input_file);
-  fseek(cinfo.input_file, 0L, 0);
+	    fseek(cinfo.input_file, 0L, 0);
 
   cinfo.output_file = NULL;	/* only wanna read */
 
-  pic24 = (byte *) NULL;
+  pic8 = pic24 = (byte *) NULL;
+  loadComments = (char *) NULL;
+
+  rmap = pinfo->r;  gmap = pinfo->g;  bmap = pinfo->b;
 
   /* Set up longjmp for error recovery out of JPEG_Error */
   rtval = setjmp(jmpState);
   if (rtval) {
     fclose(cinfo.input_file);	/* close input file */
-    return rtval;		/* no further cleanup needed */
+    if (pic8)  free(pic8);   pic8  = (byte *) NULL;
+    if (pic24) free(pic24);  pic24 = (byte *) NULL;
+    if (loadComments) free(loadComments);
+    return 0;
   }
 
+
   /* Initialize the system-dependent method pointers. */
   cinfo.methods  = &dc_methods;
   cinfo.emethods = &e_methods;
   e_methods.error_exit = JPEG_Error; /* provide my own error/message rtns */
   e_methods.trace_message = JPEG_Message;
   e_methods.trace_level = 0;	/* no tracing, thank you */
+
+  e_methods.num_warnings = 0;        /* no warnings emitted yet */
+  e_methods.first_warning_level = 0; /* display first corrupt-data warning */
+  e_methods.more_warning_level = 3;  /* but suppress additional ones */
+
   jselmemmgr(&e_methods);	/* memory allocation routines */
   dc_methods.d_ui_method_selection = d_ui_method_selection;
 
+
   /* Set up default JPEG parameters. */
   j_d_defaults(&cinfo, TRUE);
-  /* the jpeg quantizer can't handle small values of 'nc' */
-  cinfo.desired_number_of_colors = (nc<16) ? 255 : nc;
+  cinfo.desired_number_of_colors = 255;
   
+
   /* set up our progress-monitoring function */
   cinfo.methods->progress_monitor = xv_jpeg_monitor;
+
+  cinfo.methods->process_comment  = xv_process_comment;
   
+
   /* Set up to read a JFIF or baseline-JPEG file. */
   /* A smarter UI would inspect the first few bytes of the input file */
   /* to determine its type. */
   jselrjfif(&cinfo);
   
-  /* Do it! */
   jpeg_decompress(&cinfo);
 
+  fclose(cinfo.input_file);
+
+
   if (pic24) {
-    /* call XV's quantizer */
-    Conv24to8(pic24, pWIDE, pHIGH, nc);
-    free(pic24);
+    pic8 = pic24;
+    pinfo->type = PIC24;
   }
 
-  /* Close input file */
-  fclose(cinfo.input_file);
-  
-  sprintf(formatStr, "%dx%d %s JPEG. ", cinfo.image_width, cinfo.image_height, 
-	  (cinfo.out_color_space == CS_GRAYSCALE) ? "Greyscale " : "Color ");
-  
-  SetDirRButt(F_FORMAT, F_JPEG);
+  pinfo->pic = pic8;
+  pinfo->w = Width;  
+  pinfo->h = Height;
+  pinfo->frmType = F_JPEG;
+  if (cinfo.jpeg_color_space == CS_GRAYSCALE) {
+    sprintf(pinfo->fullInfo, "Greyscale JPEG. (%ld bytes)", filesize);
+    pinfo->colType = F_GREYSCALE;
+  }
+  else {
+    sprintf(pinfo->fullInfo, "Color JPEG. (%ld bytes)", filesize);
+    pinfo->colType = F_FULLCOLOR;
+  }
 
-  /* Got it! */
+  sprintf(pinfo->shrtInfo, "%ldx%ld %s JPEG. ", cinfo.image_width, 
+	  cinfo.image_height, 
+	  (cinfo.out_color_space == CS_GRAYSCALE) ? "Greyscale " : "Color ");
+
+  pinfo->comment = loadComments;
+  
+  if (pinfo->pic) return 1;
   return 0;
 }
 
@@ -719,14 +887,19 @@ static void jselrxv(cinfo)
 static int writeJFIF(fp)
      FILE *fp;
 {
-  int rtval;
-  struct compress_info_struct    cinfo;
-  struct compress_methods_struct c_methods;
+  int    i, rtval;
+  struct Compress_info_struct    cinfo;
+  struct Compress_methods_struct c_methods;
+  char  *cmt, xvcmt[256];
+
+  cmt = (char *) NULL;
 
   /* Set up longjmp for error recovery out of JPEG_Error */
   rtval = setjmp(jmpState);
-  if (rtval)
+  if (rtval) {
+    if (picComments && cmt) free(cmt);
     return rtval;		/* no further cleanup needed */
+  }
   
   /* Initialize the system-dependent method pointers. */
   cinfo.methods  = &c_methods;
@@ -734,15 +907,74 @@ static int writeJFIF(fp)
   e_methods.error_exit = JPEG_Error; /* provide my own error/message rtns */
   e_methods.trace_message = JPEG_Message;
   e_methods.trace_level = 0;	/* no tracing, thank you */
+
+  e_methods.num_warnings = 0; /* no warnings emitted yet */
+  e_methods.first_warning_level = 0; /* display first corrupt-data warning */
+  e_methods.more_warning_level = 3; /* but suppress additional ones */
+
   jselmemmgr(&e_methods);	/* memory allocation routines */
   c_methods.c_ui_method_selection = c_ui_method_selection;
 
   /* Set up default JPEG parameters. */
   j_c_defaults(&cinfo, qDial.val, FALSE);
 
+  cinfo.smoothing_factor = smDial.val;
+
   /* set up our progress-monitoring function */
   cinfo.methods->progress_monitor = xv_jpeg_monitor;
-  
+
+  sprintf(xvcmt, "%sXV %s  Quality = %d, Smoothing = %d\n",
+	  CREATOR_STR, REVDATE, qDial.val, smDial.val);
+
+  if (picComments) {   /* append XV comment */
+    char *sp;  int done;
+    i   = strlen(picComments);
+    cmt = (char *) malloc(i + strlen(xvcmt) + 2 + 1);
+    if (!cmt) FatalError("out of memory in writeJFIF()");
+
+    strcpy(cmt, picComments);
+
+    /* see if there's a line that starts with 'CREATOR: ' in the
+       comments.  If there is, rip it out. */
+
+    sp = cmt;  done = 0;
+    while (!done && *sp) {
+      if (strncmp(sp, CREATOR_STR, strlen(CREATOR_STR)) == 0) {
+	char *sp1;
+	sp1 = sp;
+	/* find end of this line */
+	while (*sp1 && *sp1 != '\n') sp1++;
+	if (*sp1 == '\n') sp1++;               /* move past \n */
+
+	/* move comments from sp1 and on down to sp */
+	xvbcopy(sp1, sp, strlen(sp1) + 1);   /* +1 to copy the trailing \0 */
+
+	done = 1;
+      }
+      else {   /* skip ahead to next line */
+	while (*sp && *sp != '\n') sp++;
+	if (*sp == '\n') sp++;
+      }
+    }
+
+    /* count # of \n's at end of comment.  
+       If none, add 2.   If one, add 1.  If two or more, add none. */
+
+    sp = cmt + strlen(cmt);
+    for (i=0; i<3 && i<strlen(cmt); i++) {
+      sp--;
+      if (*sp != '\n') break;
+    }
+
+    for ( ; i<2; i++) strcat(cmt, "\n");
+    strcat(cmt, xvcmt);
+
+    cinfo.comment_text = cmt;
+  }
+  else cinfo.comment_text = xvcmt;
+
+    
+
   /* Select input and output */
   cinfo.input_file  = NULL;
   cinfo.output_file = fp;       /* already open */
@@ -753,6 +985,8 @@ static int writeJFIF(fp)
   /* Do it! */
   jpeg_compress(&cinfo);
   
+  if (picComments && cmt) free(cmt);
+
   return 0;
 }
 
