@@ -207,18 +207,19 @@ int getpdsrec(f,buff)
 {
   static char *bp;
   static int count;
+  int c;
 
   /* read any leading CR's or LF's (sigh) */
   *buff='\0';
   bp=buff;
   count = 0;
   while (count==0) {
-    (*bp)=fgetc(f);
-    switch (*bp) {
+    c=fgetc(f);
+    switch (c) {
     case EOF: return(0);
     case '\r':
     case '\n': continue;
-    default:  ungetc((*bp),f);
+    default:  ungetc(c,f);
       count = -1;
     }
   }
@@ -226,14 +227,14 @@ int getpdsrec(f,buff)
   count=0;
   bp=buff;
   while (1) {
-    (*bp)=fgetc(f);
-    switch (*bp) {
+    c=fgetc(f);
+    switch (c) {
 
     case '\r':  *bp='\0';
-                switch ((*(bp+1))=fgetc(f)) {
+                switch (c=fgetc(f)) {
 		  case EOF:
 		  case '\n':  break;
-                  default:    ungetc(*(bp+1),f);
+                  default:    ungetc(c,f);
 		  }
                 return(count);
 
@@ -241,7 +242,7 @@ int getpdsrec(f,buff)
     
     case '\0':  return(count);
     
-    default:	count++;  bp++;
+    default:	count++;  *bp++ = c;
     }
   }
 }
@@ -278,20 +279,24 @@ int getpdsrec(f,buff)
  */
             
 /*******************************************/
-int LoadPDS(fname,nc)
-     char *fname;
-     int   nc;
+int LoadPDS(fname, pinfo)
+     char    *fname;
+     PICINFO *pinfo;
 {
+  /* returns '1' on success, '0' on failure */
 
+  byte *pic8;
   int tempnum;
   FILE	*zf;
-  static int isfixed,teco,i,j,itype,lastwasinote,
+  static int isfixed,teco,i,j,itype,
              recsize,hrecsize,irecsize,isimage,labelrecs,labelsofar,
              x,y,lpsize,lssize,samplesize,returnp,labelsize,yy;
   char	*tmp;
   char  *ftypstr;
   unsigned long filesize;
 
+  pinfo->type = PIC8;
+  pic8 = (byte *) NULL;
   isfixed = TRUE;
   returnp = isimage = FALSE;
   itype   = PDSTRASH;
@@ -309,8 +314,7 @@ int LoadPDS(fname,nc)
   zf=fopen(fname,"r");
   if (!zf) {
     SetISTR(ISTR_WARNING,"LoadPDS: can't open %s\n",fname);
-    Warning();
-    return(1);
+    return 0;
   }
 
   /* figure out the file size (for Informational Purposes Only) */
@@ -325,13 +329,12 @@ int LoadPDS(fname,nc)
 
   if ((teco=fread(rtbuff,13,1,zf)) != 1) {
     SetISTR(ISTR_WARNING,"LoadPDS: couldn't read magic #: %d,%s",teco,fname);
-    Warning();
     fclose(zf);
-    return(1);
+    return 0;
   }
 
   /* this makes sscanf so much happier */
-  rtbuff[RTBUFFSIZE-1] = 0;
+  rtbuff[RTBUFFSIZE-1] = '\0';
 
   if (strncmp(rtbuff, "NJPL1I00PDS",11) == 0) {
     itype=PDSFIXED;
@@ -345,23 +348,22 @@ int LoadPDS(fname,nc)
     itype=VICAR;
   } else {
     SetISTR(ISTR_WARNING,"LoadPDS: can't handle this file, for some reason\n");
-    Warning();
     fclose(zf);
-    return(1);
+    return 0;
   }
 
   switch (itype) {
   case PDSFIXED:
   case VICAR:
   case VIKINGFIXED:     isfixed=TRUE;
-                        SetISTR(ISTR_FORMAT, 
+                        sprintf(pinfo->fullInfo,
 				"PDS/VICAR, 8 bits per pixel. (%ld bytes)",
 				filesize);
                         break;
   case PDSVARIABLE:
   case VIKINGVARIABLE:  isfixed=FALSE;
-                        SetISTR(ISTR_FORMAT, 
-		       "Huffman-encoded PDS, 8 bits per pixel. (%ld bytes)",
+                        sprintf(pinfo->fullInfo,
+			"Huffman-encoded PDS, 8 bits per pixel. (%ld bytes)",
 				filesize);
   }
 
@@ -387,7 +389,6 @@ int LoadPDS(fname,nc)
       if (isfixed) {
 	if (getpdsrec(zf,scanbuff) == 0) {
 	  SetISTR(ISTR_WARNING,"corrupt or incomplete PDS labels (low fread)");
-	  Warning();
 	  break;
 	}
       } else {
@@ -408,9 +409,8 @@ int LoadPDS(fname,nc)
 	j=getc(zf);
 	if (j == EOF) {
 	  SetISTR(ISTR_WARNING,"LoadPDS: j is EOF\n");
-	  Warning();
 	  fclose(zf);
-	  return(1);
+	  return 0;
 	}
 	
 	teco = i + (j << 8);
@@ -418,9 +418,8 @@ int LoadPDS(fname,nc)
 
 	if (fread(scanbuff,teco,1,zf) != 1) {
 	  SetISTR(ISTR_WARNING,"LoadPDS: bad fread reading labels\n");
-	  Warning();
 	  fclose(zf);
-	  return(1);
+	  return 0;
 	}
 	
 	scanbuff[teco]='\0';
@@ -447,16 +446,15 @@ int LoadPDS(fname,nc)
 	  rtbuff[RTBUFFSIZE-1]='\0'; /* juuuust in case */
 	  SetISTR(ISTR_WARNING,
 		  "LoadPDS: unsupported record type \"%s\"\n",rtbuff);
-	  Warning();
 	  fclose(zf);
-	  return(1);
+	  return 0;
 	}
       } else if (sscanf(scanbuff,"RECORD_BYTES = %d",&recsize) == 1) {
 	/* these default to RECORD_BYTES unless explicitly set */
 	if (hrecsize == 0) hrecsize=recsize;
 	    if (irecsize == 0) irecsize=recsize;
 	continue;
-      } else if (sscanf(scanbuff,"FILE_TYPE = %s", rtbuff) != NULL) {
+      } else if (sscanf(scanbuff,"FILE_TYPE = %s", rtbuff) != 0) {
 	if (strncmp(rtbuff,"IMAGE",5) == 0) {
 	  isimage=TRUE;
 	  continue;
@@ -510,16 +508,14 @@ int LoadPDS(fname,nc)
     if ((isimage == TRUE) && (labelsize=(labelrecs * hrecsize))) {
       if (samplesize!= 8) {
 	SetISTR(ISTR_WARNING,"This image is not an 8-bit PDS file.  Sorry.\n");
-	Warning();
 	fclose(zf);
-	return(1);
+	return 0;
       }
     } else {
       SetISTR(ISTR_WARNING,
 	      "This looks like a PDS/VICAR image, but not enough.\n");
-      Warning();
       fclose(zf);
-      return(1);
+      return 0;
     }
 
     /* Some day this should be generalized to read <>8-bit files.  If I 
@@ -557,9 +553,8 @@ int LoadPDS(fname,nc)
   }
 
   if (returnp) {
-    Warning();
     fclose(zf);
-    return(1);
+    return 0;
   }
 
   /* PDS files tend to have lots of information like this in them.  The
@@ -595,24 +590,25 @@ int LoadPDS(fname,nc)
 
   switch (itype) {
   case VICAR:
-    SetISTR(ISTR_FORMAT, "VICAR, 8 bits per pixel. (%ld bytes)", filesize);
+    sprintf(pinfo->fullInfo,"VICAR, 8 bits per pixel. (%ld bytes)", filesize);
     ftypstr = "VICAR";
     rewind(zf);
     break;
 
   case PDSFIXED:
   case VIKINGFIXED:
-    SetISTR(ISTR_FORMAT, "PDS, 8 bits per pixel. (%ld bytes)", filesize);
+    sprintf(pinfo->fullInfo,"PDS, 8 bits per pixel. (%ld bytes)", filesize);
     ftypstr = "PDS";
     rewind(zf);
     break;
 
   case PDSVARIABLE:
   case VIKINGVARIABLE:
-    SetISTR(ISTR_FORMAT, "PDS, 8 bits per pixel, Huffman-encoded. (%ld bytes)",
-	    filesize);
+    sprintf(pinfo->fullInfo,
+	    "PDS, 8 bits per pixel, Huffman-encoded. (%ld bytes)", filesize);
     ftypstr = "PDS (Huffman)";
     fclose(zf);
+
 #ifndef VMS
     sprintf(pdsuncompfname,"%s/xvhuffXXXXXX", tmpdir);
     mktemp(pdsuncompfname);
@@ -624,23 +620,26 @@ int LoadPDS(fname,nc)
 #endif
 
     SetISTR(ISTR_INFO,"De-Huffmanizing '%s'...",fname);
+
     /* pdsuncomp filters to a raw file */
 #ifndef VMS
     if (tempnum=system(scanbuff)) {
+      SetISTR(ISTR_WARNING,"Unable to de-Huffmanize '%s'.",fname);
+      return 0;
+    }
 #else
     if (tempnum = !system(scanbuff)) {
-#endif
       SetISTR(ISTR_WARNING,"Unable to de-Huffmanize '%s'.",fname);
-      Warning();
-      return(1);
+      return 0;
     }
+#endif
+
 
     zf=fopen(pdsuncompfname,"r");
     if (!zf) {
       SetISTR(ISTR_WARNING,"LoadPDS: can't open uncompressed file %s\n",
 	      pdsuncompfname);
-      Warning();
-      return(1);
+      return 0;
     }
   }
 
@@ -653,7 +652,6 @@ int LoadPDS(fname,nc)
   image=(byte *)malloc(x*y);
   if (image == NULL) {
     SetISTR(ISTR_WARNING,"LoadPDS: couldn't malloc %d",x*y);
-    Warning();
     fclose(zf);
     if (isfixed == FALSE)
       unlink(pdsuncompfname);
@@ -668,43 +666,41 @@ int LoadPDS(fname,nc)
     for (yy=0; yy<y; yy++) {
       if (lpsize && ((teco=(fread(scanbuff,lpsize,1,zf))) != 1)) {
 	SetISTR(ISTR_WARNING, "LoadPDS: unexpected EOF reading prefix");
-	Warning();
 	fclose(zf);
-	return(1);
+	return 0;
       }
 
       if ((teco=(fread(image+(yy*x),x,1,zf))) != 1) {
 	SetISTR(ISTR_WARNING, "LoadPDS: unexpected EOF reading line %d",yy);
-	Warning();
 	fclose(zf);
-	return(1);
+	return 0;
       }
 
       if (lssize && ((teco=(fread(scanbuff,lssize,1,zf))) != 1)) {
 	SetISTR(ISTR_WARNING, "LoadPDS: unexpected EOF reading suffix");
-	Warning();
 	fclose(zf);
-	return(1);
+	return 0;
       }
     }
 
   } else if ((yy=fread(image,x*y,1,zf)) != 1) {
     SetISTR(ISTR_WARNING,"LoadPDS: error reading image data");
-    Warning();
     fclose(zf);
     if (itype==PDSVARIABLE || itype==VIKINGVARIABLE)
       unlink(pdsuncompfname);
 
-    return(1);
+    return 0;
   }
 
-  fclose(zf);
+    fclose(zf);
+
+
   if (isfixed == FALSE)
     unlink(pdsuncompfname);
 
-  pic = image;
-  pWIDE = x;
-  pHIGH = y;
+  pinfo->pic = image;
+  pinfo->w   = x;
+  pinfo->h    = y;
 
   /* these are grayscale, so cobble up a ramped colormap */
   /* Viking images on the CD's seem to be inverted.  Sigh */
@@ -714,19 +710,24 @@ int LoadPDS(fname,nc)
   case PDSFIXED:
   case VICAR:
     for (yy=0; yy<=255; yy++) {
-      r[yy] = g[yy] = b[yy] = yy;
+      pinfo->r[yy] = pinfo->g[yy] = pinfo->b[yy] = yy;
     }
     break;
 
   case VIKINGFIXED:
   case VIKINGVARIABLE:
     for (yy=0; yy<=255; yy++) {
-      r[yy] = g[yy] = b[yy] = (255-yy);
+      pinfo->r[yy] = pinfo->g[yy] = pinfo->b[yy] = (255-yy);
     }
   }
 
-  sprintf(formatStr, "%dx%d %s. ", pWIDE, pHIGH, ftypstr);
-  return(0);
+  pinfo->frmType = -1;   /* can't save as PDS */
+  pinfo->colType = F_GREYSCALE;
+  sprintf(pinfo->shrtInfo, "%dx%d %s. ", pinfo->w, pinfo->h, ftypstr);
+  pinfo->comment = (char *) NULL;
+
+  if (pic8) return 1;
+  return 0;
 }
 
 
